#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Unimail 0.12 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import sys, os, getpass, fileslogic, pop, poplogic, imap, imaplogic, smtp, smtplogic, configuration
import syspaths
import gettext

def notificate_search_progress(progressPercentage):
    """
    Muestra en la consola el porcentaje del proceso de búsqueda de archivos recibidos
    que han sido enviados por Unimail.

    `progressPercentage` es el porcentaje a mostrar.
    """

    sys.stdout.write("\r" + _("Searching") + "... %d%%" % progressPercentage)
    sys.stdout.flush()

def notificate_download_progress(progressPercentage):
    """
    Muestra en la consola el porcentaje del proceso de descarga de un archivo.

    `progressPercentage` es el porcentaje a mostrar.
    """

    sys.stdout.write("\r" + _("Downloading") + "... %d%%" % progressPercentage)
    sys.stdout.flush()
    
def notificate_send_progress(progressPercentage):
    """
    Muestra en la consola el porcentaje del proceso de envío de un archivo.

    `progressPercentage` es el porcentaje a mostrar.
    """

    sys.stdout.write("\r" + _("Sending") + "... %d%%" % progressPercentage)
    sys.stdout.flush()

if __name__ == "__main__":
    _ = gettext.gettext
    gettext.textdomain("unimail")
    gettext.bindtextdomain("unimail", "./languages")

    syspaths.check_dirs()

    confManag = configuration.ConfigurationManager()
    accountData = confManag.load_account_data()

    print _("Unimail 0.12 - Jose Antonio Jimenez Carmona")
    print _("Welcome to Unimail.")
    
    filesLogic = fileslogic.FilesLogic(accountData)
    quit = False
    
    while not quit:
        try:
            print ""
            print _("Options:")
            print _("--------")
            print _("1 - View inbox - IMAP")
            print _("2 - View inbox - POP")
            print _("3 - Send file")
            print _("4 - Configuration")
            print _("5 - Quit")
            print ""
            option = raw_input(_("Select an option: "))

            if option == _("1"):
                print ""
                print _("Connecting... ")
                account = imap.ImapAccount(accountData.imapServer, accountData.imapSsl, accountData.imapPort, 
                    accountData.imapUser, accountData.imapPassword)
                try:
                    account.connect()
                except:
                    raise Exception(_("Connection failure"))
                print _("Connected")
                imapLogic = imaplogic.ImapLogic(account)
                for progress in imapLogic.refresh():
                    notificate_search_progress(progress)            
                files = imapLogic.get_unimail_files_list()
                print "\n"
                print _("Inbox")
                print "-----"
    
                for i in range(0, len(files)):
                    print str(i + 1) + ": " + files[i]["file"] + _(" - From: ") + files[i]["from"]
        
                print ""
                idTxt = raw_input(_("Insert the file ID to download (0 for none): "))
                isNumber = False

                try:
                    int(idTxt)
                    isNumber = True
                except:
                    raise Exception(_("ID not valid."))

                idSelected = int(idTxt)

                if isNumber and idSelected >= 1 and idSelected <= len(files):
                    filesLogic.remove_temp_files()
                    for progress in imapLogic.download_file(files[idSelected - 1]["index"]):
                        notificate_download_progress(progress)
                    filesLogic.join_temp_files()
                    filesLogic.remove_temp_files()
                    print ""
                    print _("Done")
                elif idSelected == 0:
                    pass
                else:
                    raise Exception(_("ID not valid."))
        
                account.close()
            elif option == _("2"):
                print ""
                print _("Connecting... ")
                account = pop.PopAccount(accountData.popServer, accountData.popSsl, accountData.popPort, 
                    accountData.popUser, accountData.popPassword)
                try:
                    account.connect()
                except:
                    raise Exception(_("Connection failure"))
                print _("Connected")
                popLogic = poplogic.PopLogic(account)
                for progress in popLogic.refresh():
                    notificate_search_progress(progress)                
                files = popLogic.get_unimail_files_list()
                print "\n"
                print _("Inbox")
                print _("-----")
    
                for i in range(0, len(files)):
                    print str(i + 1) + ": " + files[i]["file"] + _(" - From: ") + files[i]["from"]
                
                print ""
                idTxt = raw_input(_("Insert the file ID to download (0 for none): "))
                isNumber = False

                try:
                    int(idTxt)
                    isNumber = True
                except:
                    raise Exception(_("ID not valid."))

                idSelected = int(idTxt)

                if isNumber and idSelected >= 1 and idSelected <= len(files):
                    filesLogic.remove_temp_files()
                    for progress in popLogic.download_file(files[idSelected - 1]["index"]):
                        notificate_download_progress(progress)
                    filesLogic.join_temp_files()
                    filesLogic.remove_temp_files()
                    print ""
                    print _("Done")
                elif idSelected == 0:
                    pass
                else:
                    raise Exception(_("ID not valid."))
        
                account.close()
            elif option == _("3"):
                print ""
                print _("Connecting...")
                account = smtp.SmtpAccount(accountData.smtpServer, accountData.smtpTls, accountData.smtpPort, 
                    accountData.smtpUser, accountData.smtpPassword)
                try:
                    account.connect()
                except:
                    raise Exception(_("Connection failure"))
                print _("Connected")
                smtpLogic = smtplogic.SmtpLogic(smtpAccount, accountData.partWaitTime)
                print ""                
                print _("Send file")
                print _("---------")
                filePath = raw_input(_("File path: "))
                fromAddress = raw_input(_("Your address: "))
                toAddress = raw_input(_('"To" address: '))
                print ""
                filesLogic.remove_temp_files()
                filesLogic.splitFile(filePath)
                for progress in smtpLogic.send_file(filePath, fromAddress, toAddress):
                    notificate_send_progress(progress)
                account.close()
                filesLogic.remove_temp_files()
                print ""
                print _("Done")
            elif option == _("4"):
                print ""
                try:
                    emailAdd = raw_input(_("E-mail address: "))
                    print _("Inbox settings:")
                    print _("---------------")
                    inboxSystem = raw_input(_("System used [imap/pop]: "))
                    print _("IMAP settings:")
                    imapServer = raw_input(_("Server: "))
                    imapSsl = (raw_input(_("Use SSL [y/n]?: ")) == "y")
                    imapPort = int(raw_input(_("Port: ")))
                    imapUser = raw_input(_("User: "))
                    imapPassword = getpass.getpass(_("Password: "))
                    print _("POP settings:")
                    popServer = raw_input(_("Server: "))
                    popSsl = (raw_input(_("Use SSL [y/n]?: ")) == "y")
                    popPort = int(raw_input(_("Port: ")))
                    popUser = raw_input(_("User: "))
                    popPassword = getpass.getpass(_("Password: "))
                    print _("Inbox folder settings:")
                    inboxDir = raw_input(_("Path: "))
                    print _("Send settings:")
                    print _("--------------")
                    print _("SMTP settings:")
                    smtpServer = raw_input(_("Server: "))
                    smtpTls = (raw_input(_("Use TLS [y/n]?: ")) == "y")
                    smtpPort = int(raw_input(_("Port: ")))
                    smtpUser = raw_input(_("User: "))
                    smtpPassword = getpass.getpass(_("Password: "))
                    print _("Parts settings:")
                    partSize = int(raw_input(_("Part size (Mb): ")))
                    partWaitTime = int(raw_input(_("Part wait time (sec.): ")))
                    accountData = configuration.AccountData(emailAdd, inboxSystem, 
                        imapServer, imapSsl, imapPort, imapUser, imapPassword,
                        popServer, popSsl, popPort, popUser, popPassword,
                        smtpServer, smtpTls, smtpPort, smtpUser, smtpPassword,
                        partSize, partWaitTime,
                        inboxDir)
                except:
                    raise Exception(_("Account data not valid"))
                confManag = configuration.ConfigurationManager()
                confManag.save_account_data(accountData)
            elif option == _("5"):
                print ""
                quit = True
            else:
                raise Exception(_("Option not valid."))
        except:
            print ""
            print _("Error: ") + str(sys.exc_info()[1])
            print ""
            raw_input(_("Press Enter to continue"))
