#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Unimail 0.12 - License: GPLv3
# Author: José Antonio Jiménez Carmona
# Project web site: https://forja.rediris.es/projects/cusl3-unimail
# Contact e-mail: joseajcarmona@gmail.com

import sys
import os
import gettext
import threading
import gobject
import gtk
import sysutils
import syspaths
import configuration
import fileslogic
import imap
import pop
import smtp
import imaplogic
import poplogic
import smtplogic
import dialaccount
import dialsendfile
import dialabout

_ = gettext.gettext
gettext.textdomain("unimail")
gettext.bindtextdomain("unimail", "./languages")
gtk.glade.textdomain("unimail")
gtk.glade.bindtextdomain("unimail", "./languages")

class WinMain:
    """
    Clase que representa la ventana principal de la interfaz de usuario gráfica.
    """

    def __init__(self):
        self.gladefile = os.path.join(sysutils.get_current_working_dir(), "winmain.glade")
        self.builder = gtk.Builder()
        self.builder.add_from_file(self.gladefile)
        self.window = self.builder.get_object("WinMain")
        self.labSessionState = self.builder.get_object("labSessionState")
        self.trvInbox = self.builder.get_object("trvInbox")
        self.progbProgress = self.builder.get_object("progbProgress")
        self.labProgress = self.builder.get_object("labProgress")
        self.statbMain = self.builder.get_object("statbMain")
        self.notbPanels = self.builder.get_object("notbPanels")
        self.butRefresh = self.builder.get_object("butRefresh")
        self.butDownload = self.builder.get_object("butDownload")
        self.mitemConfiguration = self.builder.get_object("mitemConfiguration")
        self.entFile = self.builder.get_object("entFile")
        self.entSendTo = self.builder.get_object("entSendTo")
        self.butSend = self.builder.get_object("butSend")
        self.mitemConfiguration = self.builder.get_object("mitemConfiguration")        
        self.builder.connect_signals(self)
        
        column1 = gtk.TreeViewColumn("ID", gtk.CellRendererText(), text=0)
        column1.set_resizable(True)
        self.trvInbox.append_column(column1)
        column2 = gtk.TreeViewColumn("File", gtk.CellRendererText(), text=1)
        column2.set_resizable(True)
        self.trvInbox.append_column(column2)
        column3 = gtk.TreeViewColumn("From", gtk.CellRendererText(), text=2)
        column3.set_resizable(True)
        self.trvInbox.append_column(column3)        
        self.trvInboxModel = gtk.ListStore(int, str, str)
        self.trvInbox.set_model(self.trvInboxModel)

        self.window.show_all()

        confManag = configuration.ConfigurationManager()
        self.accountData = confManag.load_account_data()
        
        self.inboxFileList = []
        self.update_interface()
        self.update_logic()
    
    def winmain_destroy(self, widget):
        gtk.main_quit()

    def update_interface(self):
        if self.accountData.address == "":
            self.labSessionState.set_text(_("Set your account configuration first"))
        else:
            self.labSessionState.set_text(self.accountData.address)

        cond1 = (self.accountData.address != "")
        cond2 = (self.accountData.inboxSystem == "imap" and self.accountData.imapServer != "") or (self.accountData.inboxSystem == "pop" and self.accountData.popServer != "")
        cond3 = (self.accountData.inboxSystem == "imap" and self.accountData.imapPort != "") or (self.accountData.inboxSystem == "pop" and self.accountData.popPort != "")
        cond4 = (self.accountData.inboxSystem == "imap" and self.accountData.imapUser != "") or (self.accountData.inboxSystem == "pop" and self.accountData.popUser != "")
        cond5 = (self.accountData.inboxSystem == "imap" and self.accountData.imapPassword != "") or (self.accountData.inboxSystem == "pop" and self.accountData.popPassword != "")

        if cond1 and cond2 and cond3 and cond4 and cond5:
            self.labSessionState.set_text(self.accountData.address)
            self.butRefresh.set_sensitive(True)
        else:
            self.butRefresh.set_sensitive(False)

    def update_logic(self):
        self.filesLogic = fileslogic.FilesLogic(self.accountData)
        
        if self.accountData.inboxSystem == "imap":
            self.emailInboxAccount = imap.ImapAccount(self.accountData.imapServer, self.accountData.imapSsl,
                self.accountData.imapPort, self.accountData.imapUser, self.accountData.imapPassword)
            self.emailInboxLogic = imaplogic.ImapLogic(self.emailInboxAccount)
        else:
            self.emailInboxAccount = pop.PopAccount(self.accountData.popServer, self.accountData.popSsl,
                self.accountData.popPort, self.accountData.popUser, self.accountData.popPassword)
            self.emailInboxLogic = poplogic.PopLogic(self.emailInboxAccount)

    def entfile_changed(self, widget):
        self.check_send_data()
    
    def entsendto_changed(self, widget):
        self.check_send_data()

    def check_send_data(self):
        cond1 = self.entFile.get_text() != ""
        cond2 = self.entSendTo.get_text() != ""
        self.butSend.set_sensitive(cond1 and cond2)

    def imitemexit_activate(self, widget):
        gtk.main_quit()

    def mitemaccount_activate(self, widget):
        dialAccount = dialaccount.DialAccount(self.accountData)
        result, data = dialAccount.run()
        if result == 0:
            self.accountData = data
            confManag = configuration.ConfigurationManager()
            confManag.save_account_data(self.accountData)
            self.update_interface()
            self.update_logic()
        
    def imitemabout_activate(self, widget):
        dialAbout = dialabout.DialAbout()
        dialAbout.run()
    
    def trvinbox_cursor_changed(self, widget):
        selection, iterator = (self.trvInbox.get_selection()).get_selected()
        cond = len(selection) > 0
        self.butDownload.set_sensitive(cond)

    def butrefresh_clicked(self, widget):
        self.refresh_inbox()        

    def butdownload_clicked(self, widget):
        selection, iterator = (self.trvInbox.get_selection()).get_selected()
        idSelected = selection.get_value(iterator, 0)
        self.download_file(idSelected)

    def refresh_inbox(self):
        menuItems = []
        menuItems.append(self.mitemConfiguration)       
        (RefreshInboxThread(self.emailInboxAccount, self.emailInboxLogic, self.labProgress, self.progbProgress, self.trvInboxModel, self.notbPanels, menuItems, self.butDownload, self.inboxFileList)).start()

    def download_file(self, idSelected):
        menuItems = []
        menuItems.append(self.mitemConfiguration)
        (DownloadFileThread(self.accountData, self.emailInboxLogic, self.inboxFileList, idSelected, self.labProgress, self.progbProgress, self.trvInboxModel, self.notbPanels, menuItems)).start()

    def butsend_clicked(self, widget):
        filePath = self.entFile.get_text()
        fromAddress = self.accountData.address
        toAddress = self.entSendTo.get_text()
        self.send_file(filePath, fromAddress, toAddress)

    def send_file(self, filePath, fromAddress, toAddress):
        menuItems = []
        menuItems.append(self.mitemConfiguration)
        (SendFileThread(self.accountData, filePath, fromAddress, toAddress, self.labProgress, self.progbProgress, self.trvInboxModel, self.notbPanels, menuItems, self.entFile, self.entSendTo)).start()

    def butbrowse_clicked(self, widget):
        dialsend_file = dialsendfile.DialSendFile()
        result, fileName = dialsend_file.run()
        if result == 0:
            self.entFile.set_text(fileName)

class RefreshInboxThread(threading.Thread):
    def __init__(self, emailInboxAccount, emailInboxLogic, labProgress, progbProgress, trvInboxModel, notbPanels, menuItems, butDownload, inboxFileList):
        super(RefreshInboxThread, self).__init__()
        self.emailInboxAccount = emailInboxAccount
        self.emailInboxLogic = emailInboxLogic
        self.labProgress = labProgress
        self.progbProgress = progbProgress
        self.trvInboxModel = trvInboxModel
        self.notbPanels = notbPanels
        self.menuItems = menuItems
        self.butDownload = butDownload
        self.inboxFileList = inboxFileList

    def process_1(self):
        self.trvInboxModel.clear()
        self.notbPanels.set_sensitive(False)

        for item in self.menuItems:
            item.set_sensitive(False)

        self.labProgress.set_text(_("Connecting..."))
        return False

    def process_2(self):
        self.labProgress.set_text(_("Searching..."))
        self.butDownload.set_sensitive(False)
        self.progbProgress.show()
        return False

    def notificate_progress(self, progress):
        self.progbProgress.set_fraction(float(progress)/100)
        return False

    def add_filelist_item(self, item):
        self.trvInboxModel.append(item)
        return False

    def process_3(self):
        self.progbProgress.hide()
        self.labProgress.set_text("")
        self.notbPanels.set_sensitive(True)

        for item in self.menuItems:
            item.set_sensitive(True)

        self.labProgress.set_text("")
        return False

    def process_4(self, fileList):
        del self.inboxFileList[:]
        for f in fileList:
            self.inboxFileList.append(f)
        return False

    def run(self):
        gobject.idle_add(self.process_1)
        self.emailInboxAccount.connect()
        gobject.idle_add(self.process_2)

        for progress in self.emailInboxLogic.refresh():
            gobject.idle_add(self.notificate_progress, progress)

        #self.inboxFileList = self.emailInboxLogic.get_unimail_files_list()
        fileList = self.emailInboxLogic.get_unimail_files_list()
        gobject.idle_add(self.process_4, fileList)
        
        #for i in range(0, len(self.inboxFileList)):
        #    gobject.idle_add(self.add_filelist_item, [i+1, self.inboxFileList[i]["file"], self.inboxFileList[i]["from"]])
        for i in range(0, len(fileList)):
            gobject.idle_add(self.add_filelist_item, [i+1, fileList[i]["file"], fileList[i]["from"]]) 
        
        gobject.idle_add(self.process_3)

class DownloadFileThread(threading.Thread):
    def __init__(self, accountData, emailInboxLogic, inboxFileList, idSelected, labProgress, progbProgress, trvInboxModel, notbPanels, menuItems):#, butDownload):
        super(DownloadFileThread, self).__init__()
        self.filesLogic = fileslogic.FilesLogic(accountData)
        self.emailInboxLogic = emailInboxLogic
        self.inboxFileList = inboxFileList
        self.idSelected = idSelected
        self.labProgress = labProgress
        self.progbProgress = progbProgress
        self.trvInboxModel = trvInboxModel
        self.notbPanels = notbPanels
        self.menuItems = menuItems
        #self.butDownload = butDownload

    def process_1(self):
        #self.mitemConfiguration.set_sensitive(False)
        for item in self.menuItems:
            item.set_sensitive(False)
        
        self.notbPanels.set_sensitive(False)
        self.labProgress.set_text(_("Downloading..."))
        return False

    def process_2(self):
        self.progbProgress.show()
        return False

    def notificate_progress(self, progress):
        self.progbProgress.set_fraction(float(progress)/100)
        return False

    def process_3(self):
        self.labProgress.set_text(_("Download completed"))
        self.progbProgress.hide()
        #self.mitemConfiguration.set_sensitive(True)
        
        for item in self.menuItems:
            item.set_sensitive(True)

        self.notbPanels.set_sensitive(True)
        return False

    def run(self):
        gobject.idle_add(self.process_1)
        self.filesLogic.remove_temp_files()
        gobject.idle_add(self.process_2)

        for progress in self.emailInboxLogic.download_file(self.inboxFileList[self.idSelected - 1]["index"]):
            gobject.idle_add(self.notificate_progress, progress)

        self.filesLogic.join_temp_files()
        self.filesLogic.remove_temp_files()
        gobject.idle_add(self.process_3)

class SendFileThread(threading.Thread):
    def __init__(self, accountData, filePath, fromAddress, toAddress, labProgress, progbProgress, trvInboxModel, notbPanels, menuItems, entFile, entSendTo):
        super(SendFileThread, self).__init__()
        self.accountData = accountData
        self.filesLogic = fileslogic.FilesLogic(accountData)
        self.filePath = filePath
        self.fromAddress = fromAddress
        self.toAddress = toAddress
        self.labProgress = labProgress
        self.progbProgress = progbProgress
        self.trvInboxModel = trvInboxModel
        self.notbPanels = notbPanels
        self.menuItems = menuItems
        self.entFile = entFile
        self.entSendTo = entSendTo

    def process_1(self):
        for item in self.menuItems:
            item.set_sensitive(False)

        self.notbPanels.set_sensitive(False)
        self.labProgress.set_text(_("Connecting..."))
        return False

    def process_2(self):
        self.progbProgress.show()
        self.labProgress.set_text(_("Sending..."))
        return False

    def notificate_progress(self, progress):
        self.progbProgress.set_fraction(float(progress)/100)
        return False
    
    def process_3(self):
        self.filesLogic.remove_temp_files()
        self.labProgress.set_text(_("Send completed"))
        self.progbProgress.hide()
        
        for item in self.menuItems:
            item.set_sensitive(True)

        self.notbPanels.set_sensitive(True)
        self.entFile.set_text("")
        self.entSendTo.set_text("")
        return False

    def run(self):
        gobject.idle_add(self.process_1)
        emailOutAccount = smtp.SmtpAccount(self.accountData.smtpServer, self.accountData.smtpTls, self.accountData.smtpPort, self.accountData.smtpUser, self.accountData.smtpPassword)
        emailOutAccount.connect()
        emailOutLogic = smtplogic.SmtpLogic(emailOutAccount, self.accountData.partWaitTime)
        self.filesLogic.remove_temp_files()
        self.filesLogic.split_file(self.filePath)
        gobject.idle_add(self.process_2)

        for progress in emailOutLogic.send_file(self.filePath, self.fromAddress, self.toAddress):
            gobject.idle_add(self.notificate_progress, progress)

        emailOutAccount.close()
        gobject.idle_add(self.process_3)
